@@ -36,7 +36,7 @@
 #include "gnome-settings-manager.h"
 #include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
-#include "gnome-settings-session.h"
+#include "gnome-settings-bus.h"
 
 #define GNOME_SESSION_DBUS_NAME      "org.gnome.SessionManager"
 #define GNOME_SESSION_CLIENT_PRIVATE_DBUS_INTERFACE "org.gnome.SessionManager.ClientPrivate"
@@ -341,7 +341,7 @@ name_acquired_handler (GDBusConnection *connection,
 {
         GDBusProxy *proxy;
 
-        proxy = gnome_settings_session_get_session_proxy ();
+        proxy = G_DBUS_PROXY (gnome_settings_bus_get_session_proxy ());
         /* Always call this first, as Setenv can only be called before
            any client registers */
         set_locale (proxy);
@@ -449,6 +449,8 @@ main (int argc, char *argv[])
 
         parse_args (&argc, &argv);
 
+        gdk_set_allowed_backends ("x11");
+
         gnome_settings_profile_start ("opening gtk display");
         if (! gtk_init_check (NULL, NULL)) {
                 g_warning ("Unable to initialize GTK+");
