@@ -403,6 +403,7 @@ gsd_orientation_manager_idle_cb (GsdOrientationManager *manager)
 
         gnome_settings_profile_start (NULL);
 
+        manager->priv->start_idle_id = 0;
         manager->priv->settings = g_settings_new (CONF_SCHEMA);
         manager->priv->orientation_lock = g_settings_get_boolean (manager->priv->settings, ORIENTATION_LOCK_KEY);
         g_signal_connect (G_OBJECT (manager->priv->settings), "changed::orientation-lock",
@@ -413,7 +414,7 @@ gsd_orientation_manager_idle_cb (GsdOrientationManager *manager)
         if (dev == NULL) {
                 g_debug ("Did not find an accelerometer");
                 gnome_settings_profile_end (NULL);
-                return FALSE;
+                return G_SOURCE_REMOVE;
         }
         manager->priv->sysfs_path = g_strdup (g_udev_device_get_sysfs_path (dev));
         g_debug ("Found accelerometer at sysfs path '%s'", manager->priv->sysfs_path);
@@ -432,7 +433,7 @@ gsd_orientation_manager_idle_cb (GsdOrientationManager *manager)
 
         gnome_settings_profile_end (NULL);
 
-        return FALSE;
+        return G_SOURCE_REMOVE;
 }
 
 gboolean
