@@ -483,9 +483,9 @@ add_printer (gchar *printer_name,
         output = g_dbus_proxy_call_sync (proxy,
                                          "PrinterAdd",
                                          g_variant_new ("(sssss)",
-                                                        printer_name ? printer_name : "",
-                                                        device_uri ? device_uri : "",
-                                                        ppd_name ? ppd_name : "",
+                                                        printer_name,
+                                                        device_uri,
+                                                        ppd_name,
                                                         info ? info : "",
                                                         location ? location : ""),
                                          G_DBUS_CALL_FLAGS_NONE,
