@@ -48,10 +48,12 @@
 #include "gsd-keygrab.h"
 #include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
+#include "gnome-settings-bus.h"
 #include "gsd-wacom-manager.h"
 #include "gsd-wacom-device.h"
 #include "gsd-wacom-oled.h"
 #include "gsd-wacom-osd-window.h"
+#include "gsd-shell-helper.h"
 
 #define GSD_WACOM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSD_TYPE_WACOM_MANAGER, GsdWacomManagerPrivate))
 
@@ -95,6 +97,12 @@ static const gchar introspection_xml[] =
 "  </interface>"
 "</node>";
 
+typedef struct
+{
+        NotifyNotification *calibration_notification;
+        GsdWacomDevice     *device;
+        guint               notification_timeout_id;
+} GsdWacomDeviceCalibration;
 
 struct GsdWacomManagerPrivate
 {
@@ -106,6 +114,8 @@ struct GsdWacomManagerPrivate
         GnomeRRScreen *rr_screen;
         GHashTable *warned_devices;
 
+        GsdShell *shell_proxy;
+
         /* button capture */
         GdkScreen *screen;
         int      opcode;
@@ -113,10 +123,6 @@ struct GsdWacomManagerPrivate
         /* Help OSD window */
         GtkWidget *osd_window;
 
-        guint notification_timeout_src_id;
-        NotifyNotification *calibration_notification;
-        GsdWacomDevice *calibration_device;
-
         /* DBus */
         GDBusNodeInfo   *introspection_data;
         GDBusConnection *dbus_connection;
@@ -129,8 +135,7 @@ static void     gsd_wacom_manager_class_init  (GsdWacomManagerClass *klass);
 static void     gsd_wacom_manager_init        (GsdWacomManager      *wacom_manager);
 static void     gsd_wacom_manager_finalize    (GObject              *object);
 
-static void     on_notification_closed (NotifyNotification *notification,
-                                        GsdWacomManager    *manager);
+static void     wacom_device_calibration_check (GsdWacomDevice  *device);
 
 static gboolean osd_window_toggle_visibility (GsdWacomManager *manager,
                                               GsdWacomDevice  *device);
@@ -261,7 +266,7 @@ wacom_set_property (GsdWacomDevice *device,
 
 	xdev = open_device (device);
 	device_set_property (xdev, gsd_wacom_device_get_tool_name (device), property);
-	xdevice_close (xdev);
+	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 }
 
 static void
@@ -443,7 +448,7 @@ set_absolute (GsdWacomDevice  *device,
 	if (gdk_error_trap_pop ())
 		g_error ("Failed to set mode \"%s\" for \"%s\".",
 			 is_absolute ? "Absolute" : "Relative", gsd_wacom_device_get_tool_name (device));
-	xdevice_close (xdev);
+	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 }
 
 static void
@@ -591,7 +596,7 @@ set_device_buttonmap (GsdWacomDevice *device,
 
 	g_free (map);
 
-	xdevice_close (xdev);
+	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 }
 
 static void
@@ -759,6 +764,30 @@ reset_touch_buttons (XDevice               *xdev,
 			       (const guchar *) actions, i);
 }
 
+/* This function does LED and OLED */
+static void
+update_pad_leds (GsdWacomDevice *device)
+{
+	GList *buttons, *l;
+
+	/* Reset all the LEDs and OLEDs*/
+	buttons = gsd_wacom_device_get_buttons (device);
+	for (l = buttons; l != NULL; l = l->next) {
+		GsdWacomTabletButton *button = l->data;
+		if (button->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED &&
+		    button->status_led != GSD_WACOM_NO_LED) {
+			set_led (device, button, 1);
+		}
+		if (button->has_oled) {
+			char *label;
+			label = g_settings_get_string (button->settings, OLED_LABEL);
+			set_oled (device, button->id, label);
+			g_free (label);
+		}
+	}
+	g_list_free (buttons);
+}
+
 static void
 reset_pad_buttons (GsdWacomDevice *device)
 {
@@ -766,7 +795,6 @@ reset_pad_buttons (GsdWacomDevice *device)
 	int nmap;
 	unsigned char *map;
 	int i, j, rc;
-	GList *buttons, *l;
 
 	/* Normal buttons */
 	xdev = open_device (device);
@@ -797,24 +825,9 @@ reset_pad_buttons (GsdWacomDevice *device)
 	reset_touch_buttons (xdev, def_touchstrip_buttons, "Wacom Strip Buttons");
 	gdk_error_trap_pop_ignored ();
 
-	xdevice_close (xdev);
+	XCloseDevice (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), xdev);
 
-	/* Reset all the LEDs and OLEDs*/
-	buttons = gsd_wacom_device_get_buttons (device);
-	for (l = buttons; l != NULL; l = l->next) {
-		GsdWacomTabletButton *button = l->data;
-                if (button->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED &&
-                    button->status_led != GSD_WACOM_NO_LED) {
-                        set_led (device, button, 1);
-                }
-		if (button->has_oled) {
-			char *label;
-			label = g_settings_get_string (button->settings, OLED_LABEL);
-			set_oled (device, button->id, label);
-			g_free (label);
-		}
-	}
-        g_list_free (buttons);
+	update_pad_leds (device);
 }
 
 static void
@@ -908,8 +921,10 @@ wacom_settings_changed (GSettings      *settings,
 	type = gsd_wacom_device_get_device_type (device);
 
 	if (g_str_equal (key, KEY_ROTATION)) {
-	        if (type != WACOM_TYPE_PAD)
-		        set_rotation (device, g_settings_get_enum (settings, key));
+		if (type != WACOM_TYPE_PAD)
+			set_rotation (device, g_settings_get_enum (settings, key));
+		else
+			update_pad_leds (device);
 	} else if (g_str_equal (key, KEY_TOUCH)) {
 	        if (type == WACOM_TYPE_TOUCH)
 			set_touch (device, g_settings_get_boolean (settings, key));
@@ -919,8 +934,9 @@ wacom_settings_changed (GSettings      *settings,
 		    type != WACOM_TYPE_TOUCH)
 			set_absolute (device, g_settings_get_boolean (settings, key));
 	} else if (g_str_equal (key, KEY_LAST_CALIBRATED_RESOLUTION)) {
-                /* We do nothing as this is used only for
-                   checking whether calibration is needed*/
+                if (type == WACOM_TYPE_STYLUS &&
+                    gsd_wacom_device_is_screen_tablet (device))
+                        wacom_device_calibration_check (device);
 	} else if (g_str_equal (key, KEY_AREA)) {
 		if (type != WACOM_TYPE_CURSOR &&
 		    type != WACOM_TYPE_PAD &&
@@ -1085,7 +1101,8 @@ notify_unknown_device (GsdWacomManager *manager, const gchar *device_name)
         notify_notification_set_app_name (notification, _("Wacom Settings"));
         notify_notification_show (notification, NULL);
 
-        g_signal_connect (notification, "closed", G_CALLBACK (on_notification_closed), manager);
+        g_signal_connect (notification, "closed",
+                          G_CALLBACK (g_object_unref), NULL);
 
         g_free (msg_body);
 }
@@ -1348,11 +1365,37 @@ switch_monitor (GsdWacomDevice *device)
 	current_monitor++;
 
 	if (current_monitor >= n_monitors)
-		current_monitor = GSD_WACOM_SET_ALL_MONITORS;
+		current_monitor = 0;
 
 	gsd_wacom_device_set_display (device, current_monitor);
 }
 
+static void
+notify_osd_for_device (GsdWacomManager *manager,
+                       GsdWacomDevice  *device)
+{
+        GdkScreen *screen;
+        gint monitor_num;
+
+        monitor_num = gsd_wacom_device_get_display_monitor (device);
+
+        if (monitor_num == GSD_WACOM_SET_ALL_MONITORS)
+                return;
+
+        screen = gdk_screen_get_default ();
+
+        if (gdk_screen_get_n_monitors (screen) == 1)
+                return;
+
+        if (manager->priv->shell_proxy == NULL)
+                manager->priv->shell_proxy = gnome_settings_bus_get_shell_proxy ();
+
+        shell_show_osd (manager->priv->shell_proxy,
+                        "input-tablet-symbolic",
+                        gsd_wacom_device_get_name (device), -1,
+                        monitor_num);
+}
+
 static const char*
 get_direction_name (GsdWacomTabletButtonType type,
                     GtkDirectionType         dir)
@@ -1474,8 +1517,10 @@ filter_button_events (XEvent          *xevent,
 
 	/* Switch monitor */
 	if (g_settings_get_enum (wbutton->settings, KEY_ACTION_TYPE) == GSD_WACOM_ACTION_TYPE_SWITCH_MONITOR) {
-		if (xiev->evtype == XI_ButtonRelease)
+		if (xiev->evtype == XI_ButtonRelease) {
 			switch_monitor (device);
+			notify_osd_for_device (manager, device);
+		}
 		return GDK_FILTER_REMOVE;
 	}
 
@@ -1538,8 +1583,7 @@ gsd_wacom_manager_idle_cb (GsdWacomManager *manager)
 }
 
 static gboolean
-check_need_for_calibration (GsdWacomManager *manager,
-                            GsdWacomDevice  *device)
+check_need_for_calibration (GsdWacomDevice  *device)
 {
         GSettings *settings;
         GVariant *variant;
@@ -1548,6 +1592,8 @@ check_need_for_calibration (GsdWacomManager *manager,
         gint monitor;
         GdkRectangle geometry;
 
+        g_debug ("Checking calibration for: %s", gsd_wacom_device_get_name (device));
+
         width = -1;
         height = -1;
 
@@ -1574,34 +1620,31 @@ check_need_for_calibration (GsdWacomManager *manager,
 }
 
 static void
-remove_notification (GsdWacomManager *manager)
+wacom_device_calibration_data_unset (GsdWacomDevice *device)
 {
-        g_clear_object (&manager->priv->calibration_notification);
-        manager->priv->calibration_device = NULL;
+        g_object_set_data (G_OBJECT (device),
+                           "gsd-wacom-calibration-data", NULL);
 }
 
 static void
-on_notification_closed (NotifyNotification *notification,
-                        GsdWacomManager    *manager)
+on_notification_closed (NotifyNotification        *notification,
+                        GsdWacomDeviceCalibration *data)
 {
-        if (notification == manager->priv->calibration_notification)
-                remove_notification (manager);
-        else
-                g_object_unref (notification);
+        wacom_device_calibration_data_unset (data->device);
 }
 
 static void
 on_notification_action (NotifyNotification *notification,
                         gchar              *action,
-                        gpointer            data)
+                        gpointer            user_data)
 {
         gboolean success;
         gchar *command;
         const gchar *device_name;
         GError *error = NULL;
-        GsdWacomManager *manager = GSD_WACOM_MANAGER (data);
+        GsdWacomDeviceCalibration *data = user_data;
 
-        device_name = gsd_wacom_device_get_name (manager->priv->calibration_device);
+        device_name = gsd_wacom_device_get_name (data->device);
 
         if (g_strcmp0 (action, "run-calibration") == 0) {
                 command = g_strdup_printf ("gnome-control-center wacom run-calibration \"%s\"",
@@ -1615,87 +1658,109 @@ on_notification_action (NotifyNotification *notification,
                 g_free (command);
         }
 
-        notify_notification_close (manager->priv->calibration_notification, NULL);
-        remove_notification (manager);
+        wacom_device_calibration_data_unset (data->device);
 }
 
 static void
-remove_calibration_notification (GsdWacomManager *manager)
+wacom_device_calibration_data_free (GsdWacomDeviceCalibration *data)
 {
-        GsdWacomManagerPrivate *priv;
+        if (data->notification_timeout_id)
+                g_source_remove (data->notification_timeout_id);
 
-        priv = manager->priv;
-
-        if (priv->calibration_notification) {
-                notify_notification_close (priv->calibration_notification, NULL);
-                g_clear_object (&priv->calibration_notification);
+        if (data->calibration_notification) {
+                notify_notification_close (data->calibration_notification, NULL);
+                g_object_unref (data->calibration_notification);
         }
-        if (manager->priv->notification_timeout_src_id > 0) {
-		g_source_remove (manager->priv->notification_timeout_src_id);
-		manager->priv->notification_timeout_src_id = 0;
-	}
-	manager->priv->calibration_device = NULL;
+
+        g_free (data);
 }
 
-static gboolean
-notify_need_for_calibration_real (gpointer data)
+static void
+wacom_device_calibration_data_update (GsdWacomDeviceCalibration *data)
 {
-        GsdWacomManager *manager = GSD_WACOM_MANAGER (data);
-        GsdWacomManagerPrivate *priv;
-        gchar *msg_body = NULL;
+        gchar *msg_body;
 
-        priv = manager->priv;
-
-        if (priv->calibration_notification) {
-                notify_notification_close (priv->calibration_notification, NULL);
-                g_clear_object (&priv->calibration_notification);
+        if (data->calibration_notification) {
+                notify_notification_close (data->calibration_notification, NULL);
+                g_object_unref (data->calibration_notification);
         }
+
         msg_body = g_strdup_printf (_("Tablet %s needs to be calibrated."),
-                                    gsd_wacom_device_get_name (priv->calibration_device));
-        priv->calibration_notification = notify_notification_new (_("Calibration needed"),
+                                    gsd_wacom_device_get_name (data->device));
+        data->calibration_notification = notify_notification_new (_("Calibration needed"),
                                                                   msg_body,
                                                                   NULL);
 
-        notify_notification_set_app_name (priv->calibration_notification,
+        notify_notification_set_app_name (data->calibration_notification,
                                           _("Wacom Settings"));
-        notify_notification_set_timeout (priv->calibration_notification,
+        notify_notification_set_timeout (data->calibration_notification,
                                          CALIBRATION_NOTIFICATION_TIMEOUT);
-        notify_notification_set_urgency (priv->calibration_notification,
+        notify_notification_set_urgency (data->calibration_notification,
                                          NOTIFY_URGENCY_NORMAL);
         /* TRANSLATORS: launches the calibration screen for the
            tablet in question */
-        notify_notification_add_action (priv->calibration_notification,
+        notify_notification_add_action (data->calibration_notification,
                                         "run-calibration",
                                         _("Calibrate"),
                                         on_notification_action,
-                                        manager,
+                                        data,
                                         NULL);
 
-        g_signal_connect (priv->calibration_notification,
+        g_signal_connect (data->calibration_notification,
                           "closed",
                           G_CALLBACK (on_notification_closed),
-                          manager);
+                          data);
 
-        notify_notification_show (priv->calibration_notification, NULL);
-
-        priv->notification_timeout_src_id = 0;
         g_free (msg_body);
-        return FALSE;
 }
 
-/* We need to have the notifications being shown from a
- * timeout function since when the monitor's size changes
- * we may get several calls to on_screen_changed_cb and
- * in the first call, the monitor's size is still not updated.
- */
-static void
-notify_need_for_calibration (GsdWacomManager *manager,
-                             GsdWacomDevice  *device)
+static GsdWacomDeviceCalibration *
+wacom_device_calibration_data_get (GsdWacomDevice *device)
 {
-        manager->priv->calibration_device = device;
-        manager->priv->notification_timeout_src_id = g_timeout_add (SHOW_CALIBRATION_TIMEOUT,
-                                                                    notify_need_for_calibration_real,
-                                                                    manager);
+        GsdWacomDeviceCalibration *data;
+
+        data = g_object_get_data (G_OBJECT (device), "gsd-wacom-calibration-data");
+
+        if (data)
+                return data;
+
+        data = g_new0 (GsdWacomDeviceCalibration, 1);
+        data->device = device;
+
+        g_object_set_data_full (G_OBJECT (device),
+                                "gsd-wacom-calibration-data", data,
+                                (GDestroyNotify) wacom_device_calibration_data_free);
+        return data;
+}
+
+static gboolean
+notify_need_for_calibration (gpointer user_data)
+{
+        GsdWacomDeviceCalibration *data = user_data;
+
+        notify_notification_show (data->calibration_notification, NULL);
+        data->notification_timeout_id = 0;
+
+        return G_SOURCE_REMOVE;
+}
+
+static void
+wacom_device_calibration_check (GsdWacomDevice  *device)
+{
+        if (check_need_for_calibration (device)) {
+                GsdWacomDeviceCalibration *data;
+
+                data = wacom_device_calibration_data_get (device);
+                wacom_device_calibration_data_update (data);
+
+                if (data->notification_timeout_id == 0) {
+                        data->notification_timeout_id = g_timeout_add (SHOW_CALIBRATION_TIMEOUT,
+                                                                       notify_need_for_calibration,
+                                                                       data);
+                }
+        } else {
+                wacom_device_calibration_data_unset (device);
+        }
 }
 
 /*
@@ -1730,14 +1795,10 @@ on_screen_changed_cb (GnomeRRScreen *rr_screen,
 		settings = gsd_wacom_device_get_settings (device);
 		/* Ignore touch devices as they do not share the same range of values for area */
 		if (type != WACOM_TYPE_TOUCH) {
-			if (gsd_wacom_device_is_screen_tablet (device) == FALSE)
+			if (gsd_wacom_device_is_screen_tablet (device) == FALSE) {
 				set_keep_aspect (device, g_settings_get_boolean (settings, KEY_KEEP_ASPECT));
-			else if (type == WACOM_TYPE_STYLUS) {
-				remove_calibration_notification (manager);
-				if (check_need_for_calibration (manager, device)) {
-					notify_need_for_calibration (manager,
-								     device);
-				}
+                        } else if (type == WACOM_TYPE_STYLUS) {
+                                wacom_device_calibration_check (device);
 			}
 
 			set_area (device, g_settings_get_value (settings, KEY_AREA));
@@ -1892,14 +1953,19 @@ gsd_wacom_manager_stop (GsdWacomManager *manager)
                 devices = gdk_device_manager_list_devices (p->device_manager, GDK_DEVICE_TYPE_SLAVE);
                 for (l = devices; l != NULL; l = l->next) {
                         GsdWacomDeviceType type;
+                        GsdWacomDevice *device;
+                        int id;
 
-                        type = gsd_wacom_device_get_device_type (l->data);
-                        if (type == WACOM_TYPE_PAD) {
-                                int id;
+                        id = gdk_x11_device_get_id (l->data);
+                        device = device_id_to_device (manager, id);
 
-                                id = get_device_id (l->data);
+                        if (!device)
+                                continue;
+
+                        type = gsd_wacom_device_get_device_type (device);
+
+                        if (type == WACOM_TYPE_PAD)
                                 grab_button (id, FALSE, manager->priv->screen);
-                        }
                 }
                 g_list_free (devices);
 
@@ -1945,9 +2011,7 @@ gsd_wacom_manager_finalize (GObject *object)
         if (wacom_manager->priv->start_idle_id != 0)
                 g_source_remove (wacom_manager->priv->start_idle_id);
 
-        if (wacom_manager->priv->notification_timeout_src_id != 0)
-                g_source_remove (wacom_manager->priv->notification_timeout_src_id);
-        remove_notification (wacom_manager);
+        g_clear_object (&wacom_manager->priv->shell_proxy);
 
         G_OBJECT_CLASS (gsd_wacom_manager_parent_class)->finalize (object);
 }
