@@ -129,10 +129,8 @@ get_sub_location (RsvgHandle *handle,
 	RsvgPositionData  position;
 	double tx, ty;
 
-	if (!rsvg_handle_get_position_sub (handle, &position, sub)) {
-		g_warning ("Failed to retrieve '%s' position", sub);
+	if (!rsvg_handle_get_position_sub (handle, &position, sub))
 		return FALSE;
-	}
 
 	tx = (double) position.x;
 	ty = (double) position.y;
@@ -783,6 +781,35 @@ osd_window_editing_button (GsdWacomOSDWindow *self)
 	return self->priv->edition_mode && gtk_widget_get_visible (self->priv->editor);
 }
 
+static RsvgHandle *
+load_rsvg_with_base (const char  *css_string,
+		     const char  *original_layout_path,
+		     GError     **error)
+{
+	RsvgHandle *handle;
+	char *dirname;
+
+	handle = rsvg_handle_new ();
+
+	dirname = g_path_get_dirname (original_layout_path);
+	rsvg_handle_set_base_uri (handle, dirname);
+	g_free (dirname);
+
+	if (!rsvg_handle_write (handle,
+				(guint8 *) css_string,
+				strlen (css_string),
+				error)) {
+		g_object_unref (handle);
+		return NULL;
+	}
+	if (!rsvg_handle_close (handle, error)) {
+		g_object_unref (handle);
+		return NULL;
+	}
+
+	return handle;
+}
+
 static void
 gsd_wacom_osd_window_update (GsdWacomOSDWindow *osd_window)
 {
@@ -859,18 +886,16 @@ gsd_wacom_osd_window_update (GsdWacomOSDWindow *osd_window)
 		                css_color_table[i].color_value);
 
 	layout_file = gsd_wacom_device_get_layout_path (osd_window->priv->pad);
+	g_debug ("Using layout path: %s", layout_file);
 	replace_string (&css_string, "layout_file", layout_file);
 
 	/* Render the SVG with the CSS applied */
-	if (osd_window->priv->handle)
-		g_object_unref (osd_window->priv->handle);
-	osd_window->priv->handle = rsvg_handle_new_from_data ((guint8 *) css_string,
-	                                                      strlen (css_string),
-	                                                      &error);
-	if (error != NULL) {
+	g_clear_object (&osd_window->priv->handle);
+	osd_window->priv->handle = load_rsvg_with_base (css_string, layout_file, &error);
+	if (osd_window->priv->handle == NULL) {
 		g_debug ("CSS applied:\n%s\n", css_string);
 		g_printerr ("RSVG error: %s\n", error->message);
-		g_clear_pointer (&error, g_error_free);
+		g_clear_error (&error);
 	}
 	g_free (css_string);
 }
@@ -1242,7 +1267,8 @@ redraw_window (GsdWacomOSDWindow *self)
 	GdkWindow *window;
 
 	window = gtk_widget_get_window (GTK_WIDGET (self));
-	gdk_window_invalidate_rect (window, NULL, FALSE);
+	if (window)
+		gdk_window_invalidate_rect (window, NULL, FALSE);
 }
 
 static GsdWacomOSDButton *
